!> \file dragging_prescr_beta_mod.f90
!< Module pour calculer le beta a partir d'une carte sur les noeuds centres.
!< Cette version est aussi fonction de la pression effective
!< calculee d'apres la hauteur au dessus de la flottaison
!<

!> \namespace dragging_prescr_beta
!! Calcule le beta a partir de vitesses de bilan
!! @note Il faut partir d'un cptr 
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!<

module dragging_beta_buoy


  use module3d_phy
  use interface_input
  implicit none
  real, dimension(nx,ny) :: Vcol_x           !< uniquement pour compatibilite
  real, dimension(nx,ny) :: Vcol_y           !<
  real, dimension(nx,ny) :: Vsl_x            !<
  real, dimension(nx,ny) :: Vsl_y            !<


  real, dimension(nx,ny) :: drag_centre      !< beta on major node (average)
                                             !< beta_centre idem est la valeur courante declare dans 3D


  real, dimension(nx,ny) :: H_buoy_init      !< hauteur au dessus de la flottaison a initialisation
  real, dimension(nx,ny) :: H_buoy_courant   !< hauteur courante au dessus de la flottaison


  real, dimension(nx,ny) :: betamoy_x        !< pour faire les moyennes glissantes
  real, dimension(nx,ny) :: betamoy_y        !<
  integer, dimension(nx,ny) :: nbvoisx       !<
  integer, dimension(nx,ny) :: nbvoisy       !<


  real :: beta_limgz                         !< when beta gt beta_limgz -> not gzmx
  real :: beta_min                           !< minimum value of beta
  real :: beta_mult                          !< coefficient of beta field
  integer  :: ill,jll,nmoy

  real :: maxi                               !< calcul correction dS a la grounding line
  real :: mini

  logical :: corr_def = .False.               !< for deformation correction (compatibility)

! declares dans le 3D (variables globales)
!-------------------------------------------
! beta_centre : valeur centree courante
! beta_mx et beta_my sont les valeurs courantes 
! drag_mx, drag_my servent dans calc_beta (remplimat-shelves-tabTu.f90) avec un autre sens 
!    et de variable de calcul ici.



contains

  !-----------------------------------------------------------------------------------------
  !> SUBROUTINE: init_dragging 
  !! Cette routine fait l'initialisation du dragging.
  !<
  subroutine init_dragging      ! Cette routine fait l'initialisation du dragging.
                                ! nouvelle version : lit les fichiers nc
    implicit none
    character(len=100) :: beta_c_file               !  beta on centered grid
    character(len=100) :: betamx_file               !  beta on staggered grid mx
    character(len=100) :: betamy_file               !  beta on staggered grid mx

    namelist/beta_prescr/beta_c_file,beta_limgz,beta_min,beta_mult

    if (itracebug.eq.1)  call tracebug(' Prescr_beta       subroutine init_dragging')

    iter_beta=0

    ! lecture des parametres du run                    
    !--------------------------------------------------------------------

    rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
428 format(A)
    read(num_param,beta_prescr)

    write(num_rep_42,428) '!___________________________________________________________' 
    write(num_rep_42,428) '!  read beta on centered grid'
    write(num_rep_42,beta_prescr)                    
    write(num_rep_42,428) '!  beta_file : nom des fichier qui contiennent les beta_c'
    write(num_rep_42,428) '!  above beta_limgz, gzmx is false'
    write(num_rep_42,428) '!  if grounded, beta_min minimum value '
    write(num_rep_42,428) '!  beta_mult : coefficient just after reading' 
    write(num_rep_42,*)
    write(num_rep_42,428) '!___________________________________________________________' 
   

    beta_c_file = trim(dirnameinp)//trim(beta_c_file)
    betamx_file = trim(dirnameinp)//trim(betamx_file)
    betamy_file = trim(dirnameinp)//trim(betamy_file)
    betamax = beta_limgz
    betamax_2d(:,:) = betamax

!  read the beta value on centered and staggered grids
!    call lect_input(1,'beta_c',1,drag_centre,beta_c_file,trim(dirnameinp)//trim(runname)//'.nc')
!    call lect_input(1,'betamx',1,drag_mx,betamx_file,trim(dirnameinp)//trim(runname)//'.nc')
!    call lect_input(1,'betamy',1,drag_my,betamy_file,trim(dirnameinp)//trim(runname)//'.nc')


    call lect_input(1,'beta_c',1,drag_centre,beta_c_file,'')

! calcule la valeur de reference de Hbuoy la hauteur au dessus de la flottaison
    call calc_buoyency(nx,ny,H,B,H_buoy_init)


! multiplication du frottement par le coefficient beta_mult

    drag_centre(:,:) = drag_centre(:,:)*beta_mult

    where (mk_init(:,:).eq.-2)                                             ! iles a probleme
       drag_centre(:,:) = 2.* abs(beta_limgz)
    end where


    where ((H(:,:).lt.0.1).and.(Bsoc(:,:).gt.0))                            ! zones actuellement non couvertes
       drag_centre(:,:) = 1000.                                             ! valeur relativement faible qui evite
    end where                                                               ! les freinages


     call  beta_c2stag(nx,ny,drag_mx,drag_my,drag_centre)         ! redistribute on staggered grid

     if  (beta_limgz.gt.0.) then
        beta_centre(:,:) =  drag_centre(:,:)
        betamx(:,:)      =  drag_mx(:,:)
        betamy(:,:)      =  drag_my(:,:)

     else if (beta_limgz.lt.0.) then                              ! attention sans doute pour plastic

        beta_centre(:,:) = - beta_limgz
        betamx(:,:)      = - beta_limgz
        betamy(:,:)      = - beta_limgz
        drag_centre(:,:) = - beta_limgz 
        drag_mx(:,:)     = - beta_limgz
        drag_my(:,:)     = - beta_limgz 
        beta_limgz = 1.
     end if



!  valeur mini que peut avoir beta (en Pa an m-1)
   call beta_min_value(nx,ny,drag_mx,drag_my,drag_centre,beta_min)     
                                     
                                      ! on peut envisager une valeur ~ 10 
                                      ! rappel : beta doit tre positif.

 

    beta_centre(:,:) =  drag_centre(:,:)                                 ! utilise aussi dans schoof



    where (drag_mx(:,:).ge.beta_limgz)
       mstream_mx(:,:) = 0
       betamx(:,:)     = beta_limgz
       drag_mx(:,:)    = beta_limgz
    elsewhere
       mstream_mx(:,:) = 1
       betamx(:,:)     = drag_mx(:,:)
    endwhere

    where (drag_my(:,:).ge.beta_limgz)
       mstream_my(:,:) = 0
       betamy(:,:)     = beta_limgz
       drag_my(:,:)    = beta_limgz
    elsewhere
       mstream_my(:,:) = 1
       betamy(:,:)     = drag_my(:,:)
    endwhere

    if (itracebug.eq.1)  call tracebug(' Prescr_beta  apres lecture')
    mstream_mx(:,:) = 0
    mstream_my(:,:) = 0
!

    call gzm_beta_prescr

    niter_nolin = 1

    return

  end subroutine init_dragging


  !________________________________________________________________________________

  !> SUBROUTINE: dragging 
  !! Defini la localisation des streams et le frottement basal
  !<
  !-------------------------------------------------------------------------
  subroutine dragging   ! defini la localisation des streams et le frottement basal

    implicit none



    if (itracebug.eq.1)  call tracebug(' beta_prescr         subroutine dragging')

    where (mstream_mx(:,:).eq.1)
       betamx(:,:) = drag_mx(:,:)
    elsewhere 
       betamx(:,:)= beta_limgz
    end where

    where (mstream_my(:,:).eq.1)
       betamy(:,:) = drag_my(:,:)
    elsewhere 
       betamy(:,:)= beta_limgz
    end where

    betamx(:,:)=drag_mx(:,:)
    betamy(:,:)=drag_my(:,:)


!   update la valeur de drag_mx et drag_my en fonction de la pression effective donnee par la hauteur au dessu de la flottaison
    call update_buoy(nx,ny,drag_centre,H_buoy_init,betamx,betamy,beta_centre,beta_min)

    debug_3D(:,:,63)  = H_buoy_courant(:,:) 
    debug_3D(:,:,101) = drag_centre(:,:) 
    debug_3D(:,:,102) = H_buoy_init(:,:) 


    return
  end subroutine dragging

  !----------------------------------------------------------------------------------
  !>SUBROUTINE: gzm_beta_prescr
  !! Calcul de gzmx
  !<

  subroutine mstream_dragging

    implicit none

    call  gzm_beta_prescr        ! determine gz et flgz et met a 0 le beta des points flottants
    
    return
  end subroutine mstream_dragging

  subroutine gzm_beta_prescr     ! attribue flgzmx et gzmx (et my)
    
    logical ::  test_Tx          ! test if there is a basal melting point in the surrounding
    logical ::  test_Ty          ! test if there is a basal melting point in the surrounding
    logical ::  test_beta_x      ! test if there is a low drag point in the surrounding
    logical ::  test_beta_y      ! test if there is a low drag point in the surrounding

    real    ::  beta_forc_fleuve ! below this value -> ice stream

!    beta_forc_fleuve = 5.e3
    beta_forc_fleuve = beta_limgz

    ! calcul de gzmx 


    gzmx(:,:)=.true.
    gzmy(:,:)=.true.
    flgzmx(:,:)=.false.
    flgzmy(:,:)=.false.


    ! points flottants : flgzmx mais pas gzmx
    do j=2,ny
       do i=2,nx
          if (flot(i,j).and.(flot(i-1,j))) then
             flotmx(i,j)=.true.
             flgzmx(i,j)=.true.
             gzmx(i,j)=.false.
             betamx(i,j)=0.
          end if
          if (flot(i,j).and.(flot(i,j-1))) then
             flotmy(i,j)=.true.
             flgzmy(i,j)=.true.
             gzmy(i,j)=.false.
             betamy(i,j)=0.
          end if
       end do
    end do

    if (itracebug.eq.1)  call tracebug(' apres criteres flot')

    if (itracebug.eq.1) write(num_tracebug,*) 'gzmx', gzmx(127,330) 

    !--------- autres criteres

    ! points poses
    gzmx(:,:)=gzmx(:,:).and.(betamx(:,:).lt.beta_limgz)   !  Pas de calcul pour les points
    gzmy(:,:)=gzmy(:,:).and.(betamy(:,:).lt.beta_limgz)   !  au fort beta



    ! critere (lache) sur la temperature
    do j = 2, ny-1
       do i =2, nx-1 

!   test s'il y a un point tempere dans les environs
          test_Tx = any( ibase( i-1:i   , j-1:j+1 ).gt.1)
          test_Ty = any( ibase( i-1:i+1 , j-1:j   ).gt.1)

!   test s'il y a un point low drag dans les environs
          test_beta_x = any( drag_centre( i-1:i   , j-1:j+1 ) .lt. beta_forc_fleuve )
          test_beta_y = any( drag_centre( i-1:i+1 , j-1:j   ) .lt. beta_forc_fleuve )

!   critere pour gzmx

! Attention : les deux lignes suivants sont en commentaire pour voir l'effet

!          gzmx(i,j) =  gzmx(i,j) .and. (test_Tx.or.test_beta_x)
!          gzmy(i,j) =  gzmy(i,j) .and. (test_Ty.or.test_beta_y)

       end do
    end do


    if (itracebug.eq.1)  call tracebug(' apres autres criteres ')
    if (itracebug.eq.1) write(num_tracebug,*) 'gzmx', gzmx(127,330) 

    flgzmx(:,:) = flgzmx(:,:) .or. gzmx(:,:)
    flgzmy(:,:) = flgzmy(:,:) .or. gzmy(:,:)

    where ((.not.flotmx(:,:)).and.(.not.gzmx(:,:)))
       betamx(:,:) = beta_limgz
    end where

    where ((.not.flotmy(:,:)).and.(.not.gzmy(:,:)))
       betamy(:,:) = beta_limgz
    end where

	fleuvemx(:,:)=gzmx(:,:)
	fleuvemy(:,:)=gzmy(:,:)

    
  end subroutine gzm_beta_prescr


  !----------------------------------------------------------------------------------
  ! Routines qui permettent des manip simples sur beta.



!______________________________________________________________________________________
!>SUBROUTINE:   beta_min_value 
!! valeur mini que peut avoir beta (en Pa an m-1) 
!<

  subroutine beta_min_value(nxx,nyy,R_mx,R_my,R_centre,val_betamin)
    implicit none
    integer                                 ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(inout) ::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(inout) ::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(inout) ::  R_centre               ! valeur du beta sur maille centree   
    real                    , intent(in)    :: val_betamin     ! valeur mini que peut avoir beta (en Pa an m-1)

    R_centre(:,:) = max(R_centre(:,:),val_betamin)
    R_mx(:,:)     = max(R_mx(:,:),val_betamin)
    R_my(:,:)     = max(R_my(:,:),val_betamin)

    where(flot(:,:))
       drag_centre(:,:) = 0.
    end where

  end subroutine beta_min_value
!______________________________________________________________________________________
!>SUBROUTINE:   beta_stag2c 
!! average the staggered values on central ones
!<

  subroutine beta_stag2c(nxx,nyy,R_mx,R_my,R_centre)      ! average the staggered values on central ones

    implicit none
    integer                                ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(in)   ::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(in)   ::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(out)  ::  R_centre               ! valeur du beta sur maille centree   
    
    R_centre(:,:)=0.
    do j=2,ny-1
       do i=2,nx-1
          R_centre(i,j)= ((R_mx(i,j)+R_mx(i+1,j)) &
               + (R_my(i,j)+R_my(i,j+1)))*0.25
       end do
    end do
  end subroutine beta_stag2c
!______________________________________________________________________________________
!>SUBROUTINE:   beta_c2stag_min  
!! redistribute central values on staggered grid
!<

  subroutine beta_c2stag(nxx,nyy,R_mx,R_my,R_centre)              ! redistribute central values on staggered grid

    implicit none
    integer                                ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(out)  ::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(out)  ::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(in)   ::  R_centre               ! valeur du beta sur maille centree

    do j=2,nyy 
       do i=2,nxx
          R_mx(i,j)= (R_centre(i-1,j)+R_centre(i,j))*0.5
          R_my(i,j)= (R_centre(i,j-1)+R_centre(i,j))*0.5
       end do
    end do

  end subroutine beta_c2stag
!______________________________________________________________________________________
!>SUBROUTINE:   beta_c2stag_min  
!! redistribute central values on staggered grid
!<

  subroutine beta_c2stag_min(nxx,nyy,R_mx,R_my,R_centre)              ! redistribute central values on staggered grid

    implicit none
    integer                                ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(inout)::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(inout)::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(in)   ::  R_centre               ! valeur du beta sur maille centree

    do j=2,nyy 
       do i=2,nxx
          R_mx(i,j)= min(R_mx(i,j),(R_centre(i-1,j)+R_centre(i,j))*0.5)
          R_my(i,j)= min(R_my(i,j),(R_centre(i,j-1)+R_centre(i,j))*0.5)
       end do
    end do

  end subroutine beta_c2stag_min

!>SUBROUTINE:  update_buoy
!! fait changer la valeur centree enfonction de la hauteur au dessus de la flottaison.
!< on pressuppose beta = beta0 * Hbuoy

  subroutine update_buoy(nxx,nyy,Ref_centre,Ref_Hbuoy,R_mx,R_my,R_centre,val_betamin)
    implicit none
    integer                                ::  nxx,nyy                ! dimension des tableaux
    real, dimension(nxx,nyy), intent(in)   ::  Ref_centre             ! valeur de reference dragging centre
    real, dimension(nxx,nyy), intent(in)   ::  Ref_Hbuoy              ! valeur de reference hauteur au dessus de la flottaison
    real, dimension(nxx,nyy), intent(out)  ::  R_mx                   ! valeur du beta sur maille mx
    real, dimension(nxx,nyy), intent(out)  ::  R_my                   ! valeur du beta sur maille my
    real, dimension(nxx,nyy), intent(out)  ::  R_centre               ! valeur du beta sur maille centree
    real                    , intent(in)   :: val_betamin             ! valeur mini que peut avoir beta (en Pa an m-1)

    call calc_buoyency(nxx,nyy,H,B,H_buoy_courant)    



    where (Ref_Hbuoy(:,:).gt.0.)
!       R_centre(:,:) = Ref_centre(:,:)*max(H_buoy_courant(:,:),1.)/max(Ref_Hbuoy(:,:),1.)


! avec une dpendance non linaire
       R_centre(:,:) = Ref_centre(:,:)*(max((max(H_buoy_courant(:,:),1.)/max(Ref_Hbuoy(:,:),1.)),0.01))**(1./3.)



 
    elsewhere  (Ref_Hbuoy(:,:).le.0.)   ! flottant (0.) ou eventuellement pas de glace
       R_centre(:,:) = Ref_centre(:,:)

    end where

! valeur mini

    R_centre(:,:) = max(R_centre(:,:),val_betamin)

! si flottaison drag = 0.

    where(flot(:,:))
       drag_centre(:,:) = 0.
    end where

! remet sur les noeuds staggered

    do j=2,nyy 
       do i=2,nxx
          R_mx(i,j)= min(R_mx(i,j),(R_centre(i-1,j)+R_centre(i,j))*0.5)
          R_my(i,j)= min(R_my(i,j),(R_centre(i,j-1)+R_centre(i,j))*0.5)
       end do
    end do

! valeur mini
    R_mx(:,:)     = max(R_mx(:,:),val_betamin)
    R_my(:,:)     = max(R_my(:,:),val_betamin)


! par contre, je ne reteste pas sur beta_limgz pour eviter d'avoir des points qui sautent

  end subroutine update_buoy


!>SUBROUTINE:  calc_buoyency
!< calcule la hauteur au dessus de la flottaison.

  subroutine calc_buoyency(nxx,nyy,H1,B1,H_buoy)  

    implicit none
    integer                                ::  nxx,nyy              ! dimension des tableaux
    real, dimension(nxx,nyy), intent(in)   ::  H1                   ! epaisseur
    real, dimension(nxx,nyy), intent(in)   ::  B1                   ! base de la glace (attention : pas Bsoc)
    real, dimension(nxx,nyy), intent(out)  ::  H_buoy


    where ( sealevel_2d(:,:)-B1(:,:).le.0.)             ! socle au dessus du niveau des mers
       H_buoy(:,:) = H1(:,:)

    elsewhere
       H_buoy(:,:) = H1(:,:)-row/ro*(sealevel_2d(:,:)-B1(:,:))
       
    end where
 
  end subroutine calc_buoyency


end module dragging_beta_buoy

